# A function which filters options which starts with "-" from $argv.
function _swift_swiftlint_preprocessor
    set -l results
    for i in (seq (count $argv))
        switch (echo $argv[$i] | string sub -l 1)
            case '-'
            case '*'
                echo $argv[$i]
        end
    end
end

function _swift_swiftlint_using_command
    set -l currentCommands (_swift_swiftlint_preprocessor (commandline -opc))
    set -l expectedCommands (string split " " $argv[1])
    set -l subcommands (string split " " $argv[2])
    if [ (count $currentCommands) -ge (count $expectedCommands) ]
        for i in (seq (count $expectedCommands))
            if [ $currentCommands[$i] != $expectedCommands[$i] ]
                return 1
            end
        end
        if [ (count $currentCommands) -eq (count $expectedCommands) ]
            return 0
        end
        if [ (count $subcommands) -gt 1 ]
            for i in (seq (count $subcommands))
                if [ $currentCommands[(math (count $expectedCommands) + 1)] = $subcommands[$i] ]
                    return 1
                end
            end
        end
        return 0
    end
    return 1
end

complete -c swiftlint -n '_swift_swiftlint_using_command "swiftlint analyze"' -l config -d 'The path to one or more SwiftLint configuration files, evaluated as a parent-child hierarchy.'
complete -c swiftlint -n '_swift_swiftlint_using_command "swiftlint analyze"' -l fix -l autocorrect -d 'Correct violations whenever possible.'
complete -c swiftlint -n '_swift_swiftlint_using_command "swiftlint analyze"' -l format -d 'Should reformat the Swift files using the same mechanism used by Xcode (via SourceKit).
Only applied with `--fix`/`--autocorrect`.'
complete -c swiftlint -n '_swift_swiftlint_using_command "swiftlint analyze"' -l use-alternative-excluding -d 'Use an alternative algorithm to exclude paths for `excluded`, which may be faster in some cases.'
complete -c swiftlint -n '_swift_swiftlint_using_command "swiftlint analyze"' -l use-script-input-files -d 'Read SCRIPT_INPUT_FILE* environment variables as files.'
complete -c swiftlint -n '_swift_swiftlint_using_command "swiftlint analyze"' -l strict -d 'Upgrades warnings to serious violations (errors).'
complete -c swiftlint -n '_swift_swiftlint_using_command "swiftlint analyze"' -l lenient -d 'Downgrades serious violations to warnings, warning threshold is disabled.'
complete -c swiftlint -n '_swift_swiftlint_using_command "swiftlint analyze"' -l force-exclude -d 'Exclude files in config `excluded` even if their paths are explicitly specified.'
complete -c swiftlint -n '_swift_swiftlint_using_command "swiftlint analyze"' -l benchmark -d 'Save benchmarks to `benchmark_files.txt` and `benchmark_rules.txt`.'
complete -c swiftlint -n '_swift_swiftlint_using_command "swiftlint analyze"' -l reporter -d 'The reporter used to log errors and warnings.'
complete -c swiftlint -n '_swift_swiftlint_using_command "swiftlint analyze"' -l in-process-sourcekit -d 'Use the in-process version of SourceKit.'
complete -c swiftlint -n '_swift_swiftlint_using_command "swiftlint analyze"' -l output -d 'The file where violations should be saved. Prints to stdout by default.'
complete -c swiftlint -n '_swift_swiftlint_using_command "swiftlint analyze"' -l progress -d 'Show a live-updating progress bar instead of each file being processed.'
complete -c swiftlint -n '_swift_swiftlint_using_command "swiftlint analyze"' -l quiet -d 'Don\'t print status logs like \'Analyzing <file>\' & \'Done analyzing\'.'
complete -c swiftlint -n '_swift_swiftlint_using_command "swiftlint analyze"' -l compiler-log-path -d 'The path of the full xcodebuild log to use when running AnalyzerRules.'
complete -c swiftlint -n '_swift_swiftlint_using_command "swiftlint analyze"' -l compile-commands -d 'The path of a compilation database to use when running AnalyzerRules.'
complete -c swiftlint -n '_swift_swiftlint_using_command "swiftlint analyze"' -s h -l help -d 'Show help information.'
complete -c swiftlint -n '_swift_swiftlint_using_command "swiftlint docs"' -s h -l help -d 'Show help information.'
complete -c swiftlint -n '_swift_swiftlint_using_command "swiftlint generate-docs"' -l path -d 'The directory where the documentation should be saved'
complete -c swiftlint -n '_swift_swiftlint_using_command "swiftlint generate-docs"' -l config -d 'The path to a SwiftLint configuration file'
complete -c swiftlint -n '_swift_swiftlint_using_command "swiftlint generate-docs"' -l enabled -s e -d 'Only show enabled rules'
complete -c swiftlint -n '_swift_swiftlint_using_command "swiftlint generate-docs"' -l disabled -s d -d 'Only show disabled rules'
complete -c swiftlint -n '_swift_swiftlint_using_command "swiftlint generate-docs"' -l correctable -s c -d 'Only display correctable rules'
complete -c swiftlint -n '_swift_swiftlint_using_command "swiftlint generate-docs"' -s h -l help -d 'Show help information.'
complete -c swiftlint -n '_swift_swiftlint_using_command "swiftlint lint"' -l config -d 'The path to one or more SwiftLint configuration files, evaluated as a parent-child hierarchy.'
complete -c swiftlint -n '_swift_swiftlint_using_command "swiftlint lint"' -l fix -l autocorrect -d 'Correct violations whenever possible.'
complete -c swiftlint -n '_swift_swiftlint_using_command "swiftlint lint"' -l format -d 'Should reformat the Swift files using the same mechanism used by Xcode (via SourceKit).
Only applied with `--fix`/`--autocorrect`.'
complete -c swiftlint -n '_swift_swiftlint_using_command "swiftlint lint"' -l use-alternative-excluding -d 'Use an alternative algorithm to exclude paths for `excluded`, which may be faster in some cases.'
complete -c swiftlint -n '_swift_swiftlint_using_command "swiftlint lint"' -l use-script-input-files -d 'Read SCRIPT_INPUT_FILE* environment variables as files.'
complete -c swiftlint -n '_swift_swiftlint_using_command "swiftlint lint"' -l strict -d 'Upgrades warnings to serious violations (errors).'
complete -c swiftlint -n '_swift_swiftlint_using_command "swiftlint lint"' -l lenient -d 'Downgrades serious violations to warnings, warning threshold is disabled.'
complete -c swiftlint -n '_swift_swiftlint_using_command "swiftlint lint"' -l force-exclude -d 'Exclude files in config `excluded` even if their paths are explicitly specified.'
complete -c swiftlint -n '_swift_swiftlint_using_command "swiftlint lint"' -l benchmark -d 'Save benchmarks to `benchmark_files.txt` and `benchmark_rules.txt`.'
complete -c swiftlint -n '_swift_swiftlint_using_command "swiftlint lint"' -l reporter -d 'The reporter used to log errors and warnings.'
complete -c swiftlint -n '_swift_swiftlint_using_command "swiftlint lint"' -l in-process-sourcekit -d 'Use the in-process version of SourceKit.'
complete -c swiftlint -n '_swift_swiftlint_using_command "swiftlint lint"' -l output -d 'The file where violations should be saved. Prints to stdout by default.'
complete -c swiftlint -n '_swift_swiftlint_using_command "swiftlint lint"' -l progress -d 'Show a live-updating progress bar instead of each file being processed.'
complete -c swiftlint -n '_swift_swiftlint_using_command "swiftlint lint"' -l use-stdin -d 'Lint standard input.'
complete -c swiftlint -n '_swift_swiftlint_using_command "swiftlint lint"' -l quiet -d 'Don\'t print status logs like \'Linting <file>\' & \'Done linting\'.'
complete -c swiftlint -n '_swift_swiftlint_using_command "swiftlint lint"' -l silence-deprecation-warnings -d 'Don\'t print deprecation warnings.'
complete -c swiftlint -n '_swift_swiftlint_using_command "swiftlint lint"' -l cache-path -d 'The directory of the cache used when linting.'
complete -c swiftlint -n '_swift_swiftlint_using_command "swiftlint lint"' -l no-cache -d 'Ignore cache when linting.'
complete -c swiftlint -n '_swift_swiftlint_using_command "swiftlint lint"' -l enable-all-rules -d 'Run all rules, even opt-in and disabled ones, ignoring `only_rules`.'
complete -c swiftlint -n '_swift_swiftlint_using_command "swiftlint lint"' -s h -l help -d 'Show help information.'
complete -c swiftlint -n '_swift_swiftlint_using_command "swiftlint reporters"' -s h -l help -d 'Show help information.'
complete -c swiftlint -n '_swift_swiftlint_using_command "swiftlint rules"' -l config -d 'The path to a SwiftLint configuration file'
complete -c swiftlint -n '_swift_swiftlint_using_command "swiftlint rules"' -l enabled -s e -d 'Only show enabled rules'
complete -c swiftlint -n '_swift_swiftlint_using_command "swiftlint rules"' -l disabled -s d -d 'Only show disabled rules'
complete -c swiftlint -n '_swift_swiftlint_using_command "swiftlint rules"' -l correctable -s c -d 'Only display correctable rules'
complete -c swiftlint -n '_swift_swiftlint_using_command "swiftlint rules"' -l verbose -s v -d 'Display full configuration details'
complete -c swiftlint -n '_swift_swiftlint_using_command "swiftlint rules"' -l config-only -d 'Print only the YAML configuration(s)'
complete -c swiftlint -n '_swift_swiftlint_using_command "swiftlint rules"' -l default-config -d 'Print default configuration(s)'
complete -c swiftlint -n '_swift_swiftlint_using_command "swiftlint rules"' -s h -l help -d 'Show help information.'
complete -c swiftlint -n '_swift_swiftlint_using_command "swiftlint version"' -l verbose -d 'Display full version info'
complete -c swiftlint -n '_swift_swiftlint_using_command "swiftlint version"' -s h -l help -d 'Show help information.'
complete -c swiftlint -n '_swift_swiftlint_using_command "swiftlint" "analyze docs generate-docs lint reporters rules version help"' -l version -d 'Show the version.'
complete -c swiftlint -n '_swift_swiftlint_using_command "swiftlint" "analyze docs generate-docs lint reporters rules version help"' -s h -l help -d 'Show help information.'
complete -c swiftlint -n '_swift_swiftlint_using_command "swiftlint" "analyze docs generate-docs lint reporters rules version help"' -f -a 'analyze' -d 'Run analysis rules'
complete -c swiftlint -n '_swift_swiftlint_using_command "swiftlint" "analyze docs generate-docs lint reporters rules version help"' -f -a 'docs' -d 'Open SwiftLint documentation website in the default web browser'
complete -c swiftlint -n '_swift_swiftlint_using_command "swiftlint" "analyze docs generate-docs lint reporters rules version help"' -f -a 'generate-docs' -d 'Generates markdown documentation for selected group of rules'
complete -c swiftlint -n '_swift_swiftlint_using_command "swiftlint" "analyze docs generate-docs lint reporters rules version help"' -f -a 'lint' -d 'Print lint warnings and errors'
complete -c swiftlint -n '_swift_swiftlint_using_command "swiftlint" "analyze docs generate-docs lint reporters rules version help"' -f -a 'reporters' -d 'Display the list of reporters and their identifiers'
complete -c swiftlint -n '_swift_swiftlint_using_command "swiftlint" "analyze docs generate-docs lint reporters rules version help"' -f -a 'rules' -d 'Display the list of rules and their identifiers'
complete -c swiftlint -n '_swift_swiftlint_using_command "swiftlint" "analyze docs generate-docs lint reporters rules version help"' -f -a 'version' -d 'Display the current version of SwiftLint'
complete -c swiftlint -n '_swift_swiftlint_using_command "swiftlint" "analyze docs generate-docs lint reporters rules version help"' -f -a 'help' -d 'Show subcommand help information.'
